--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2021

Author: GtX | Andy
Date: 19.01.2021

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


local loadError
local numInserted = 0

local sleeperCabVehicles
local sleeperCabStoreItems

local buildId = 1
local versionString = "0.0.0.0"

local modName = g_currentModName
local modDirectory = g_currentModDirectory

local function getIsVehicleSleeper(vehicle)
    return vehicle.spec_sleeperCab ~= nil and vehicle:getIsSleeper()
end

local function loadGlobalStoreItems(xmlFilePath, globalStoreItems)
    local numLoaded = 0

    if xmlFilePath ~= nil and fileExists(xmlFilePath) and globalStoreItems ~= nil then
        local xmlFile = loadXMLFile("VehiclesDataXML", xmlFilePath)

        if xmlFile ~= nil and xmlFile ~= 0 then
            local i = 0
            while true do
                local key = string.format("vehicleSleeperCab.globalStoreItems.storeItem(%d)", i)
                if not hasXMLProperty(xmlFile, key) then
                    break
                end

                local xmlFilename = getXMLString(xmlFile, key .. "#xmlFilename")

                if xmlFilename ~= nil then
                    local storeitem = g_storeManager:getItemByXMLFilename(NetworkUtil.convertFromNetworkFilename(xmlFilename))

                    if storeitem ~= nil and globalStoreItems[storeitem.xmlFilenameLower] == nil then
                        local translation = StringUtil.getVectorNFromString(getXMLString(xmlFile, key .. ".camera#translation"), 3)
                        local rotation = StringUtil.getRadiansFromString(getXMLString(xmlFile, key .. ".camera#rotation"), 3)

                        if (translation ~= nil and #translation == 3) and (rotation ~= nil and #rotation == 3) then
                            globalStoreItems[storeitem.xmlFilenameLower] = {
                                fovy = getXMLFloat(xmlFile, key .. ".camera#fovy"),
                                nearClip = getXMLFloat(xmlFile, key .. ".camera#nearClip"),
                                farClip = getXMLFloat(xmlFile, key .. ".camera#farClip"),
                                configuredVersion = Utils.getNoNil(getXMLString(xmlFile, key .. "#version"), "1.0.0.0"),
                                translation = translation,
                                rotation = rotation
                            }

                            numLoaded = numLoaded + 1
                        end
                    end
                end

                i = i + 1
            end

            delete(xmlFile)
        end
    end

    return numLoaded
end

local function finalizeVehicleTypes(vehicleTypeManager)
    local specializationName = string.format("%s.sleeperCab", modName)
    local specializationObject = g_specializationManager:getSpecializationObjectByName(specializationName)

    if specializationObject ~= nil then
        for typeName, typeEntry in pairs(vehicleTypeManager.vehicleTypes) do
            if specializationObject.prerequisitesPresent(typeEntry.specializations) then
                g_vehicleTypeManager:addSpecialization(typeName, specializationName)
                numInserted = numInserted + 1
            end
        end
    end
end

local function playerNew(player, superFunc, isServer, isClient)
    local instance = superFunc(player, isServer, isClient)

    if instance.inputInformation ~= nil then
        function onInputSleep(playerObject, actionName, inputValue, callbackState, isAnalog, isMouse, deviceCategory)
            local vehicle = g_currentMission.interactiveVehicleInRange

            if vehicle and vehicle.spec_sleeperCab ~= nil then
                vehicle:openSleepDialog(playerObject)
            end
        end

        instance.inputInformation.registrationList[InputAction.VEHICLE_SLEEP] = {
            text = g_i18n:getText("ui_inGameSleep"),
            triggerAlways = false,
            triggerDown = true,
            eventId = "",
            textVisibility = false,
            triggerUp = false,
            callback = onInputSleep,
            activeType = Player.INPUT_ACTIVE_TYPE.STARTS_DISABLED
        }
    end

    return instance
end

local function playerUpdateActionEvents(player)
    local sleeperCabInput = player.inputInformation.registrationList[InputAction.VEHICLE_SLEEP]

    if sleeperCabInput ~= nil then
        local vehicleSleepInputState = player.canEnterVehicle and getIsVehicleSleeper(g_currentMission.interactiveVehicleInRange)

        g_inputBinding:setActionEventTextVisibility(sleeperCabInput.eventId, vehicleSleepInputState)
        g_inputBinding:setActionEventActive(sleeperCabInput.eventId, vehicleSleepInputState)
    end
end

local function farmGetSleepCamera(farm, superFunc)
    if g_currentMission.controlledVehicle ~= nil then
        return 0 -- Stop players camera being stuck under map if they are in a vehicle just leave camera alone @Giants
    end

    if not farm.isSpectator and sleeperCabVehicles ~= nil then
        local vehicleInRange, vehicleInRangeDistance = nil, math.huge
        local closestVehicle, closestVehicleDistance = nil, math.huge

        for _, vehicle in pairs(sleeperCabVehicles) do
            if not vehicle.isBroken and getIsVehicleSleeper(vehicle) and g_currentMission.accessHandler:canFarmAccessOtherId(farm.farmId, vehicle:getOwnerFarmId()) then
                local vehicleDistance = vehicle:getDistanceToNode(g_currentMission.player.rootNode)

                if vehicleDistance < vehicleInRangeDistance then
                    vehicleInRangeDistance = vehicleDistance
                    vehicleInRange = vehicle
                else
                    local distanceToVehicle = calcDistanceFrom(vehicle.rootNode, g_currentMission.player.rootNode)

                    if distanceToVehicle < closestVehicleDistance then
                        closestVehicleDistance = distanceToVehicle
                        closestVehicle = vehicle
                    end
                end
            end
        end

        -- If the player is next to a sleeper cab vehicle use it
        if vehicleInRange ~= nil then
            return vehicleInRange:getSleepCamera() or 0
        end

        -- If there is a house use it otherwise use closest sleeper vehicle
        if closestVehicle ~= nil then
            local sleepCamera = superFunc(farm)

            if sleepCamera ~= nil and sleepCamera ~= 0 then
                return sleepCamera
            end

            return closestVehicle:getSleepCamera() or 0
        end
    end

    -- All else fails use original function
    return superFunc(farm)
end

local function sleepManagerStartSleep(sleepManager)
    local player = g_currentMission.player

    if sleeperCabVehicles ~= nil then
        for _, vehicle in pairs(sleeperCabVehicles) do
            if not vehicle.isBroken and vehicle.spec_sleeperCab ~= nil then
                vehicle:onStartSleep() -- Notify vehicle sleep is starting
            end
        end
    end

    if player ~= nil and player.isEntered and player.isClient then
        player:removeActionEvents() -- Do not show input bindings when sleeping

        if player.isLightActive then
            player:setLightIsActive(false) -- Disable torch
        end
    end
end

local function sleepManagerStopSleep(sleepManager)
    local player = g_currentMission.player

    if player ~= nil and player.isEntered and player.isClient then
        player:registerActionEvents() -- Restore input bindings
        player:updateActionEvents() -- Correct input texts
    end
end

local function loadMapFinished(mission)
    if numInserted > 0 then
        sleeperCabVehicles = {}
        sleeperCabStoreItems = {}

        local numLoaded = loadGlobalStoreItems(modDirectory .. "vehicleSleeperCab.xml", sleeperCabStoreItems)

        mission.sleeperCabVehicles = sleeperCabVehicles
        mission.sleeperCabStoreItems = sleeperCabStoreItems

        if numLoaded > 0 then
            print(string.format("  Info: [SleeperCabVehicles] - Version %s has been loaded into %d Global Vehicles successfully.", versionString, numLoaded))
        end
    end
end

local function init()
    local mod = g_modManager:getModByName(modName)
    versionString = tostring(mod.version) or versionString

    if mod.modName == "FS19_VehicleSleeperCab" or mod.modName == "FS19_VehicleSleeperCab_update" and mod.author:len() == 3 then
        g_specializationManager:addSpecialization("sleeperCab", "SleeperCab", modDirectory .. "scripts/specializations/SleeperCab.lua")
        VehicleTypeManager.finalizeVehicleTypes = Utils.prependedFunction(VehicleTypeManager.finalizeVehicleTypes, finalizeVehicleTypes)

        Player.new = Utils.overwrittenFunction(Player.new, playerNew)
        Player.updateActionEvents = Utils.appendedFunction(Player.updateActionEvents, playerUpdateActionEvents)

        Farm.getSleepCamera = Utils.overwrittenFunction(Farm.getSleepCamera, farmGetSleepCamera)

        SleepManager.startSleep = Utils.prependedFunction(SleepManager.startSleep, sleepManagerStartSleep)
        SleepManager.stopSleep = Utils.prependedFunction(SleepManager.stopSleep, sleepManagerStopSleep)

        BaseMission.loadMapFinished = Utils.appendedFunction(BaseMission.loadMapFinished, loadMapFinished)
    else
        loadError = {
            author = mod.author,
            startUpdateTime = 2400,

            update = function(self, dt)
                self.startUpdateTime = self.startUpdateTime - dt

                if self.startUpdateTime < 0 then
                    local text = string.format(g_i18n:getText("VSC_loadError", modName), modName, self.author)

                    if g_dedicatedServerInfo == nil then
                        if not g_gui:getIsGuiVisible() then
                            g_gui:showYesNoDialog({
                                title = string.format("%s - Version %s", mod.title, versionString),
                                text = text,
                                dialogType = DialogElement.TYPE_LOADING,
                                callback = self.openModHubLink,
                                target = nil,
                                yesText = g_i18n:getText("button_ok"),
                                noText = g_i18n:getText("button_modHubDownload")
                            })
                        end
                    else
                        print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0" .. "\n")
                        self.openModHubLink(true)
                    end
                end
            end,

            openModHubLink = function(ignore)
                if ignore == false then
                    local language = g_languageShort
                    local link = "mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0"
                    if language == "de" or language == "fr" then
                        link = "mods.php?lang=" .. language .. "&country=be&title=fs2019&filter=org&org_id=129652&page=0"
                    end

                    openWebFile(link, "")
                end

                removeModEventListener(loadError)
                loadError = nil
            end
        }

        addModEventListener(loadError)
    end
end

init()
