--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2021

Author: GtX | Andy
Date: 19.01.2021

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


SleeperCab = {}
SleeperCab.MOD_NAME = g_currentModName

function SleeperCab.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Enterable, specializations) and
        not SpecializationUtil.hasSpecialization(Locomotive, specializations) and
        not SpecializationUtil.hasSpecialization(Rideable, specializations)
end

function SleeperCab.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "openSleepDialog", SleeperCab.openSleepDialog)
    SpecializationUtil.registerFunction(vehicleType, "getSleepCamera", SleeperCab.getSleepCamera)
    SpecializationUtil.registerFunction(vehicleType, "getIsSleeper", SleeperCab.getIsSleeper)
    SpecializationUtil.registerFunction(vehicleType, "onStartSleep", SleeperCab.onStartSleep)
end

function SleeperCab.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad",   SleeperCab)
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", SleeperCab)
end

function SleeperCab:onLoad(savegame)
    self.spec_sleeperCab = self[string.format("spec_%s.sleeperCab", SleeperCab.MOD_NAME)]

    local spec = self.spec_sleeperCab

    spec.isSleeper = false
    spec.deactivateEngineOnSleep = true
    spec.deactivateLightsOnSleep = true

    if g_currentMission.sleeperCabVehicles ~= nil and g_currentMission.sleeperCabStoreItems ~= nil then
        spec.isSleeper = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.enterable.sleeperCab#isSleeper"), spec.isSleeper)

        if spec.isSleeper then
            spec.sleepCameraNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.enterable.sleeperCab#cameraNode"), self.i3dMappings)

            spec.deactivateEngineOnSleep = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.enterable.sleeperCab#deactivateEngineOnSleep"), spec.deactivateEngineOnSleep)
            spec.deactivateLightsOnSleep = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.enterable.sleeperCab#deactivateLightsOnSleep"), spec.deactivateLightsOnSleep)

            g_currentMission.sleeperCabVehicles[self] = self
        else
            local cameraData = g_currentMission.sleeperCabStoreItems[self.configFileName:lower()]

            if cameraData ~= nil and cameraData.translation ~= nil and cameraData.rotation ~= nil then
                local sleepCameraNode = createCamera("sleepCamera", math.rad(cameraData.fovy or 60), cameraData.nearClip or 0.1, cameraData.farClip or 5000)

                setTranslation(sleepCameraNode, unpack(cameraData.translation))
                setRotation(sleepCameraNode, unpack(cameraData.rotation))

                link(self.components[1].node, sleepCameraNode)

                spec.isSleeper = true
                spec.sleepCameraNode = sleepCameraNode

                g_currentMission.sleeperCabVehicles[self] = self
            end
        end
    end
end

function SleeperCab:onDelete()
    if g_currentMission.sleeperCabVehicles ~= nil then
        g_currentMission.sleeperCabVehicles[self] = nil
    end
end

function SleeperCab:openSleepDialog()
    if self:getIsSleeper() then
        g_sleepManager:showDialog()
    end
end

function SleeperCab:onStartSleep()
    local spec = self.spec_sleeperCab

    if spec ~= nil and spec.isSleeper then
        if spec.deactivateEngineOnSleep and self.getIsMotorStarted ~= nil and self:getIsMotorStarted() then
            self:stopMotor(true)
        end

        if spec.deactivateLightsOnSleep and self.deactivateLights ~= nil then
            self:deactivateLights()
        end
    end
end

function SleeperCab:getSleepCamera()
    local spec = self.spec_sleeperCab

    if spec ~= nil and spec.sleepCameraNode ~= nil then
        return spec.sleepCameraNode
    end

    spec = self.spec_enterable

    if spec ~= nil and spec.cameras[1] ~= nil then
        return spec.cameras[1].cameraNode
    end

    return 0
end

function SleeperCab:getIsSleeper()
    return self.spec_sleeperCab.isSleeper
end
